/******************************************************************
** KMyGLCubeSaver - (c) 2010-2012 by Felice Murolo, all rigths reserved
** Author: Felice Murolo, Salerno, Italia
** eMail: linuxboy@RE-MO-VEfel.hopto.org
**
** GNU Lesser General Public License Usage
** This program and it's sources may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
**/

#include "glwidget.h"
#include <GL/glu.h>;
#include <QImage>
#include <time.h>

// directory where you must put your images
QString dirpath=QDir::homePath()+"/.myglcubesaver";
QString file;
int current,currenttic,entrynum;

// CONFIGURAZIONE
// tempo permanenza fotografie, in secondi
// viene determinata dal modulo myglcubesaver.cpp
extern int changeticsec;
extern bool randomview;
extern bool xmove,ymove,zmove;
extern float xrot,yrot,zrot;

// converte il parametro changeticsec da secondi a tick di 50 millisecondi
// 50 ms = tempo di aggiornamento della grafica GL, settato nel timer GLWidget::m_timer
int changetic;

// include una xpm di default
//#include "logo.xpm"
#include "linux_inside.xpm"

class GLWidgetImpl : public GLWidget
{
private:
	GLfloat rotx, roty, rotz;
	GLuint texture[1];

	void loadGLTextures()
	{
	   QImage t;
		QImage b(dummy);

		changetic=changeticsec*20;

		QDir dir(dirpath);
		QStringList filter;
      filter << "*.jpg" << "*.bmp" << "*.png" << "*.jpeg" << "*.gif";
      dir.setNameFilters(filter);
		QStringList entry = dir.entryList();
		entrynum=entry.size();
		if (entrynum)
		{
		  file = entry.at(current);
		  if (randomview)
		  {
			 file = entry.at(getrand(entrynum-1));
		  }

		  if ( !b.load( dirpath+"/"+file ) )
		  {
			 b = QImage( 1024, 1024, QImage::Format_ARGB32 );
			 b.fill( QColor(0,255,0).rgb() );
		  }
		}
		
		t = QGLWidget::convertToGLFormat( b );
		glGenTextures( 1, &texture[0] );
		glBindTexture( GL_TEXTURE_2D, texture[0] );
		glTexImage2D( GL_TEXTURE_2D, 0, 3, t.width(), t.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, t.bits() );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	}
	
public:
	GLWidgetImpl( QWidget *parent=0, char *name=0 ) : GLWidget( 50, parent, name )
	{
		rotx = roty = rotz = 0.0f;
	}
	
protected:
  
  void initializeGL()
	{
		loadGLTextures();
		
		QDir d(dirpath);
		if (!d.exists())
		{
		  d.mkdir(dirpath);
		}
		
		glEnable(GL_TEXTURE_2D);
		glShadeModel(GL_SMOOTH);

		glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		glClearDepth(1.0f);

		glEnable(GL_DEPTH_TEST);
		glDepthFunc(GL_LEQUAL);

		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
		currenttic=0;
		current=0;
	}
	
	void resizeGL( int width, int height )
	{
  	height = height?height:1;

		glViewport( 0, 0, (GLint)width, (GLint)height );

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}
	
	void paintGL()
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glLoadIdentity();

		glTranslatef(0.0f,0.0f,-5.0f);

		if (xmove) glRotatef(rotx,1.0f,0.0f,0.0f);
		if (ymove) glRotatef(roty,0.0f,1.0f,0.0f);
		if (zmove) glRotatef(rotz,0.0f,0.0f,1.0f);  

		glBindTexture(GL_TEXTURE_2D, texture[0]);

		glBegin(GL_QUADS);
		// Front Face
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		// Back Face
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		// Top Face
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
		// Bottom Face
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		// Right face
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		// Left Face
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
		glEnd();
	}
	
	void timeOut()
	{
		rotx += xrot;
		roty += yrot;
		rotz += zrot;

		currenttic++;
		if (currenttic >= changetic)
		{
		  currenttic=0;
		  current++;
		  if (current>=entrynum && entrynum>0)
		  {
			 current=0;
		  }
		  loadGLTextures();
		}
		updateGL();
	}
	 // restituisce valori casuali tra 0 e max, estremi compresi
	 int getrand(int max)
	 {
		int r;
		//formula rand()%(min-max+1) + min
		qsrand(time(NULL));
		r=qrand()%(max+1);
		return(r);
	 }
};
