/******************************************************************
** KMyGLCubeSaver - (c) 2010-2012 by Felice Murolo, all rigths reserved
** Author: Felice Murolo, Salerno, Italia
** eMail: linuxboy@RE-MO-VEfel.hopto.org
**
** GNU Lesser General Public License Usage
** This program and it's sources may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
**/

#include <stdlib.h>
#include <QLabel>
#include <QLayout>
#include <QFrame>
#include <QGridLayout>
#include <kapplication.h>
#include <klocale.h>
#include <kconfig.h>
#include <kcolordialog.h>
#include <kcolorbutton.h>
#include <kglobal.h>
#include "myglcubesaver.h"
#include <kaboutdata.h>
#include <kdeversion.h>
//#include "myglcubesaver.moc"
#include "glwidgetimpl.h"

GLWidgetImpl *glw;

// PROGRAM ABOUT DATA
QByteArray version="1.01";
const char *author="Felice Murolo";
const char *respfor="All";
QByteArray email="linuxboy@fel.hopto.org";
QByteArray web="http://nothing.for.now";

// Configurazione di default
int changeticsec=10;
bool randomview=TRUE;
bool xmove=TRUE;
bool ymove=TRUE;
bool zmove=TRUE;
float xrot=0.3f,yrot=0.2f,zrot=0.4f;

// libkscreensaver interface
class KMyGLCubeSaverInterface : public KScreenSaverInterface
{
public:
    virtual KAboutData* aboutData()
    {
		KAboutData *k = new KAboutData( "kmyglcubescrn.kss", "kmyglcubescrn", ki18n( "KMyGLCubeScreen" ),
                               version, ki18n( "MyGLCube Screen Saver" ) );
		k->setShortDescription(ki18n("KDE4 ScreenSaver that shows a rotating cube on which are represented on your favorite photos"));
        k->setLicense(KAboutData::License_LGPL_V2);
		k->addAuthor(ki18n(author),ki18n(respfor),email,web);
		return(k);
    }

    virtual KScreenSaver* create( WId id )
    {
        return new KMyGLCubeSaver( id );
    }

    virtual QDialog* setup()
    {
        return new KMyGLCubeSetup();
    }
};

int main( int argc, char *argv[] )
{
    KMyGLCubeSaverInterface kss;
    return kScreenSaverMain( argc, argv, kss );
}

//-----------------------------------------------------------------------------
// dialog to setup screen saver parameters
//
KMyGLCubeSetup::KMyGLCubeSetup( QWidget *parent, const char *name )
	: KDialog( parent )
{
  setObjectName( name );
  setModal( true );
  setCaption( i18n( "Setup MyGLCube Screen Saver" ) );
  setButtons( Ok | Cancel );
  showButtonSeparator( true );

	readSettings();

	QFrame *main = new QFrame( this );
   setMainWidget( main );
	QGridLayout *grid = new QGridLayout(main);
	grid->setSpacing(spacingHint());
	grid->setMargin(0);

	QLabel *label = new QLabel( i18n("Use the hidden directory '.myglcubesaver' in your home dir to put your own photos or images."), main );
	label->setWordWrap(TRUE);
	grid->addWidget(label, 0, 0,1,4);

	QLabel *label1 = new QLabel( i18n("Shows randomly"), main );
	grid->addWidget(label1, 1, 0);
	QCheckBox *cb_randomview = new QCheckBox(this);
	cb_randomview->setCheckState(randomview ? Qt::Checked : Qt::Unchecked);
	grid->addWidget(cb_randomview, 1, 1);
	connect(cb_randomview,SIGNAL(stateChanged(int)),this,SLOT(slot_cb_randomview(int)));

	QLabel *label2 = new QLabel( i18n("X-axis movement"), main );
	grid->addWidget(label2, 2, 0);
	QCheckBox *cb_xmove = new QCheckBox(this);
	cb_xmove->setCheckState(xmove ? Qt::Checked : Qt::Unchecked);
	grid->addWidget(cb_xmove, 2, 1);
	connect(cb_xmove,SIGNAL(stateChanged(int)),this,SLOT(slot_cb_xmove(int)));
   QSpinBox *sp_xrot = new QSpinBox(this);
	sp_xrot->setRange(1,10);
   sp_xrot->setValue(int(xrot*10));
	grid->addWidget(sp_xrot, 2, 2);
	connect(sp_xrot,SIGNAL(valueChanged(int)),this,SLOT(slot_sp_xrot(int)));


	QLabel *label3 = new QLabel( i18n("Y-axis movement"), main );
	grid->addWidget(label3, 3, 0);
	QCheckBox *cb_ymove = new QCheckBox(this);
	cb_ymove->setCheckState(ymove ? Qt::Checked : Qt::Unchecked);
	grid->addWidget(cb_ymove, 3, 1);
	connect(cb_ymove,SIGNAL(stateChanged(int)),this,SLOT(slot_cb_ymove(int)));
   QSpinBox *sp_yrot = new QSpinBox(this);
	sp_yrot->setRange(1,10);
   sp_yrot->setValue(int(yrot*10));
	grid->addWidget(sp_yrot, 3, 2);
	connect(sp_yrot,SIGNAL(valueChanged(int)),this,SLOT(slot_sp_yrot(int)));
	
	QLabel *label4 = new QLabel( i18n("Z-axis movement"), main );
	grid->addWidget(label4, 4, 0);
	QCheckBox *cb_zmove = new QCheckBox(this);
	cb_zmove->setCheckState(zmove ? Qt::Checked : Qt::Unchecked);
	grid->addWidget(cb_zmove, 4, 1);
	connect(cb_zmove,SIGNAL(stateChanged(int)),this,SLOT(slot_cb_zmove(int)));
   QSpinBox *sp_zrot = new QSpinBox(this);
	sp_zrot->setRange(1,10);
   sp_zrot->setValue(int(zrot*10));
	grid->addWidget(sp_zrot, 4, 2);
	connect(sp_zrot,SIGNAL(valueChanged(int)),this,SLOT(slot_sp_zrot(int)));

	QLabel *label5 = new QLabel( i18n("Image change time (sec)"), main );
	grid->addWidget(label5, 5, 0);
   QSpinBox *sp_changeticsec = new QSpinBox(this);
	sp_changeticsec->setRange(1,60);
   sp_changeticsec->setValue(changeticsec);
	grid->addWidget(sp_changeticsec, 5, 1);
	connect(sp_changeticsec,SIGNAL(valueChanged(int)),this,SLOT(slot_sp_changeticsec(int)));

	preview = new QWidget( main );
	preview->setFixedSize( 220, 165 );
	QPalette palette;
	palette.setColor( preview->backgroundRole(), Qt::black );
	preview->setPalette(palette);
	preview->setAutoFillBackground(true);
	preview->show();    // otherwise saver does not get correct size
	saver = new KMyGLCubeSaver( preview->winId() );
	grid->addWidget(preview, 1, 3, 5, 1);

	grid->setRowStretch( 6, 10 );
	grid->setRowStretch( 7, 20 );

	setMinimumSize( sizeHint() );
	connect(this,SIGNAL(okClicked()),this,SLOT(slotOk()));
}

// read settings from config file
void KMyGLCubeSetup::readSettings()
{
	KConfigGroup config(KGlobal::config(), "Settings");
	randomview = config.readEntry( "RandomView", randomview );
	xmove = config.readEntry( "XMove", xmove );
	ymove = config.readEntry( "YMove", ymove );
	zmove = config.readEntry( "ZMove", zmove );
	changeticsec = config.readEntry( "ChangeSeconds", changeticsec );
	xrot = config.readEntry( "XRot", xrot );
	yrot = config.readEntry( "YRot", yrot );
	zrot = config.readEntry( "ZRot", zrot );
}

// Ok pressed - save settings and exit
void KMyGLCubeSetup::slotOk()
{
	KConfigGroup config(KGlobal::config(), "Settings");
 	config.writeEntry( "RandomView", randomview );
 	config.writeEntry( "XMove", xmove );
 	config.writeEntry( "YMove", ymove );
 	config.writeEntry( "ZMove", zmove );
 	config.writeEntry( "ChangeSeconds", changeticsec );
 	config.writeEntry( "XRot", xrot );
 	config.writeEntry( "YRot", yrot );
 	config.writeEntry( "ZRot", zrot );
	config.sync();

 	accept();
}

// The value into spinbox is changed
void KMyGLCubeSetup::slot_sp_changeticsec(int v)
{
  changeticsec = v;
}

// The value into spinbox is changed
void KMyGLCubeSetup::slot_sp_xrot(int v)
{
  xrot = (float)v/10;
}
// The value into spinbox is changed
void KMyGLCubeSetup::slot_sp_yrot(int v)
{
  yrot = (float)v/10;
}
// The value into spinbox is changed
void KMyGLCubeSetup::slot_sp_zrot(int v)
{
  zrot = (float)v/10;
}


// The state into checkbox is changed
void KMyGLCubeSetup::slot_cb_xmove(int v)
{
  xmove = ( (v ==Qt::Unchecked) ? FALSE : TRUE);
}
void KMyGLCubeSetup::slot_cb_ymove(int v)
{
  ymove = ( (v ==Qt::Unchecked) ? FALSE : TRUE);
}
void KMyGLCubeSetup::slot_cb_zmove(int v)
{
  zmove = ( (v ==Qt::Unchecked) ? FALSE : TRUE);
}
void KMyGLCubeSetup::slot_cb_randomview(int v)
{
  randomview = ( (v ==Qt::Unchecked) ? FALSE : TRUE);
}


//-----------------------------------------------------------------------------


KMyGLCubeSaver::KMyGLCubeSaver( WId id ) : KScreenSaver( id )
{
	readSettings();
	blank();
   glw = new GLWidgetImpl();
   embed(glw);
   glw->show();
}

KMyGLCubeSaver::~KMyGLCubeSaver()
{
}

// read configuration settings from config file
void KMyGLCubeSaver::readSettings()
{
	KConfigGroup config(KGlobal::config(), "Settings");
	randomview = config.readEntry( "RandomView", randomview );
	xmove = config.readEntry( "XMove", xmove );
	ymove = config.readEntry( "YMove", ymove );
	zmove = config.readEntry( "ZMove", zmove );
	changeticsec = config.readEntry( "ChangeSeconds", changeticsec );
	xrot = config.readEntry( "XRot", xrot );
	yrot = config.readEntry( "YRot", yrot );
	zrot = config.readEntry( "ZRot", zrot );
}

void KMyGLCubeSaver::blank()
{
	QPalette palette;
	palette.setColor( backgroundRole(), QColor(0,0,0) );
	setPalette(palette);
}

